/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.identityaggregator.client;

import eu.semiramis.identityaggregator.commons.Provider;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.parse.XMLParserException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

/**
 * Client configuration
 * 
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
class ClientConfig {

    protected Properties properties = null;
    public String BASEPATH;
    public String SEMIRAMIS_CLIENT_ID;
    public String SEMIRAMIS_CLIENT_DOMAIN;
    public String SEMIRAMIS_IA_ID;
    public String SEMIRAMIS_ORIGIN_ISSUER;
    public String API_PKCS12_PATH;
    public String API_PKCS12_PASSWORD;
    public String API_PKCS12_CERT_ALIAS;
    public String API_PRIVATE_KEY_PASSWORD;
    public String RELYING_PARTY_FILE_PATH;
    public boolean ENABLE_LOG_TO_AUDIT_SYSTEM = false;
    public String AUDIT_SERVER_CONFIG_PATH;
    public String RESPONSE_URL;
    public String LOG_CONFIG_PATH;
    public String TEMPLATE = "/templates/saml2-post-simplesign-binding.vm";
    public Map<String, Provider> Providers = new HashMap<String, Provider>();

    ClientConfig(String configFilePath) throws FileNotFoundException, XMLParserException {
        loadConfigFile(configFilePath);
    }

    protected void loadConfigFile(String configFilePath) throws FileNotFoundException, XMLParserException {
        try {
            properties = new Properties();
            properties.loadFromXML(new FileInputStream(configFilePath));
            Enumeration<Object> keys = properties.keys();
            for (; keys.hasMoreElements();) {
                String actual = (String) keys.nextElement();
                setProperty(actual, properties.getProperty(actual));
            }
        } catch (IOException ex) {
            Logger.getLogger(ClientConfig.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    protected void setProperty(String current, String property) throws FileNotFoundException, XMLParserException {
        if (current.equals("BASEPATH")) {
            BASEPATH = property;
        }
        if (current.contains("_PATH")) {
            if (!property.startsWith("/")) {
                property = properties.getProperty("BASEPATH") + property;
            }
        }
        if (current.equals("SEMIRAMIS_CLIENT_ID")) {
            SEMIRAMIS_CLIENT_ID = property;
        }
        if (current.equals("SEMIRAMIS_CLIENT_DOMAIN")) {
            SEMIRAMIS_CLIENT_DOMAIN = property;
        }
        if (current.equals("SEMIRAMIS_IA_ID")) {
            SEMIRAMIS_IA_ID = property;
        }
        if (current.equals("SEMIRAMIS_ORIGIN_ISSUER")) {
            SEMIRAMIS_ORIGIN_ISSUER = property;
        }
        if (current.equals("API_PKCS12_PATH")) {
            API_PKCS12_PATH = property;
        }
        if (current.equals("API_PKCS12_PASSWORD")) {
            API_PKCS12_PASSWORD = property;
        }
        if (current.equals("API_PKCS12_CERT_ALIAS")) {
            API_PKCS12_CERT_ALIAS = property;
        }
        if (current.equals("API_PRIVATE_KEY_PASSWORD")) {
            API_PRIVATE_KEY_PASSWORD = property;
        }
        if (current.equals("LOG_CONFIG_PATH")) {
            LOG_CONFIG_PATH = property;
        }
        if (current.equals("RESPONSE_URL")) {
            RESPONSE_URL = property;
        }
        if (current.equals("TEMPLATE")) {
            TEMPLATE = property;
        }
        if (current.equals("RELYING_PARTY_FILE_PATH")) {
            RELYING_PARTY_FILE_PATH = property;
            //read SP config file
            BasicParserPool b = new BasicParserPool();
            FileInputStream f = new FileInputStream(property);
            Document d = b.parse(f);

            NodeList nodeList = d.getElementsByTagName("RP");
            for (int i = 0; i < nodeList.getLength(); i++) {
                Provider provider = new Provider(nodeList.item(i), properties.getProperty("BASEPATH"));
                Providers.put(provider.getId(), provider);
            }
        }
        if (current.equals("ENABLE_LOG_TO_AUDIT_SYSTEM")) {
            ENABLE_LOG_TO_AUDIT_SYSTEM = Boolean.parseBoolean(property);
        }
        if (current.equals("AUDIT_SERVER_CONFIG_PATH")) {
            AUDIT_SERVER_CONFIG_PATH = property;
        }

    }
}
