/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator.commons;

import eu.semiramis.commons.SAMLAttribute;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;
import java.util.Map;
import org.opensaml.xml.XMLObject;

/**
 * Defines an attribute
 * 
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class Attribute {

    SAMLAttribute samlAttribute;

    public Attribute(SAMLAttribute samlAttribute) {
        this.samlAttribute = samlAttribute;
    }

    public Attribute(String id) {
        this.samlAttribute = new SAMLAttribute(id);
    }

    public Attribute(String id, String value) {
        this.samlAttribute = new SAMLAttribute(id, value);
    }

    public Attribute(String id, String friendlyName, String value) {
        this.samlAttribute = new SAMLAttribute(id, friendlyName, value);
    }

    public Attribute(String id, String friendlyName, Map<String, String> extendedInfo, String value) {
        this.samlAttribute = new SAMLAttribute(id, friendlyName, extendedInfo, value);
    }

    public Attribute(String id, String friendlyName, Map<String, String> extendedInfo, XMLObject value) {
        this.samlAttribute = new SAMLAttribute(id, friendlyName, extendedInfo, value);
    }

    public String getId() {
        return this.samlAttribute.getName();
    }

    public void setId(String id) {
        this.samlAttribute.setName(id);
    }

    public String getFriendlyName() {
        return samlAttribute.getFriendlyName();
    }

    public void setFriendlyName(String friendlyName) {
        this.samlAttribute.setFriendlyName(friendlyName);
    }

    public Map<String, String> getExtendedInfo() {
        return this.samlAttribute.getUnknownAttributes();
    }

    public void setExtendedInfo(Map<String, String> extendedInfo) {
        this.samlAttribute.setUnknownAttributes(extendedInfo);
    }

    public void addExtendedInfo(String key, String value) {
        this.samlAttribute.addUnknownAttribute(key, value);
    }

    public String getExtendedInfo(String key) {
        return this.samlAttribute.getUnknownAttribute(key);
    }

    public void removeExtendedInfo(String key) {
        this.samlAttribute.removeUnknownAttribute(key);
    }

    public boolean isRequired() {
        return this.samlAttribute.isRequired();
    }

    public void setRequired(boolean required) {
        this.samlAttribute.setRequired(required);
    }

    public boolean isValueCyphered() {
        return this.samlAttribute.isValueCyphered();
    }

    public String getValue() {
        return this.samlAttribute.getValue();
    }

    public void setValue(String value) {
        this.samlAttribute.setValue(value);
    }

    public String getDecryptedValue(String pkcs12Path, String certPassword, String certAlias, String privateKeyPassword) throws Exception {
        return this.samlAttribute.getDecryptedValue(pkcs12Path, certPassword, certAlias, privateKeyPassword);
    }

    public String getDecryptedValue(PublicKey publicKey, PrivateKey privateKey) throws Exception {
        return this.samlAttribute.getDecryptedValue(publicKey, privateKey);
    }

    public void setEncryptedValue(String value, String certPath) throws Exception {
        this.samlAttribute.setEncryptedValue(value, certPath);
    }

    public void setEncryptedValue(String value, X509Certificate cert) throws Exception {
        this.samlAttribute.setEncryptedValue(value, cert);
    }

    public void encryptValue(String certPath) throws Exception {
        this.samlAttribute.encryptValue(certPath);
    }

    public void encryptValue(X509Certificate cert) throws Exception {
        this.samlAttribute.encryptValue(cert);
    }

    public void decryptValue(PublicKey publicKey, PrivateKey privateKey) throws Exception {
        this.samlAttribute.decryptValue(publicKey, privateKey);
    }

    public XMLObject getXMLObjectValue() {
        return this.samlAttribute.getXMLObjectValue();
    }

    public SAMLAttribute toSAMLAttribute() {
        return this.samlAttribute;
    }
}
