/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator.commons;

import eu.semiramis.commons.SAMLUtil;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.AttributeStatement;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.XMLParserException;
import org.opensaml.xml.util.AttributeMap;

/**
 * Attribute Token
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class AttributeToken extends Token {

    public AttributeToken(Assertion assertion) throws XMLParserException, UnmarshallingException {
        Assertion assertionCopy = SAMLUtil.cloneAssertion(assertion);
        setAssertion(assertionCopy);
    }

    public AttributeToken(String assertion) throws UnmarshallingException, XMLParserException {
        setAssertion((Assertion) SAMLUtil.stringToSAML(assertion));
    }

    public List<Attribute> getAttributes() {
        List<Attribute> attributes = new ArrayList<Attribute>();
        Iterator<AttributeStatement> it = assertion.getAttributeStatements().iterator();
        while (it.hasNext()) {
            AttributeStatement attributeStatement = it.next();
            Iterator<org.opensaml.saml2.core.Attribute> it2 = attributeStatement.getAttributes().iterator();
            while (it2.hasNext()) {
                org.opensaml.saml2.core.Attribute attribute = it2.next();
                Attribute attr = null;
                Map<String, String> unknownAttributes = null;
                AttributeMap samlUnknownAttributes = attribute.getUnknownAttributes();
                if (samlUnknownAttributes != null) {
                    unknownAttributes = new HashMap<String, String>();
                    for (Map.Entry<QName, String> samlUnknownAttribute : samlUnknownAttributes.entrySet()) {
                        unknownAttributes.put(samlUnknownAttribute.getKey().getLocalPart(), samlUnknownAttribute.getValue());
                    }
                }
                attr = new Attribute(attribute.getName(), attribute.getFriendlyName(), unknownAttributes, attribute.getAttributeValues().get(0));
                attributes.add(attr);
            }
        }
        return attributes;
    }
}
