/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator.commons;

import eu.semiramis.commons.SAMLUtil;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.Audience;
import org.opensaml.saml2.core.AudienceRestriction;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.XMLParserException;

/**
 * Authentication Token
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class AuthenticationToken extends Token {

    public AuthenticationToken(Assertion assertion) throws XMLParserException, UnmarshallingException {
        Assertion assertionCopy = SAMLUtil.cloneAssertion(assertion);
        setAssertion(assertionCopy);
    }

    public AuthenticationToken(String assertion) throws XMLParserException, UnmarshallingException {
        setAssertion((Assertion) SAMLUtil.stringToSAML(assertion));
    }

    public Conditions getConditions() {
        Conditions conditions = new Conditions();
        org.opensaml.saml2.core.Conditions samlConditions = assertion.getConditions();
        if (samlConditions != null) {
            conditions.setNotBefore(samlConditions.getNotBefore());
            conditions.setNotOnOrAfter(samlConditions.getNotOnOrAfter());
            if (samlConditions.getOneTimeUse() != null) {
                conditions.setOneTimeUse(true);
            }
            if (samlConditions.getAudienceRestrictions() != null) {
                List<String> audienceRestrictionList = new ArrayList<String>();
                Iterator<AudienceRestriction> it = samlConditions.getAudienceRestrictions().iterator();
                while (it.hasNext()) {
                    AudienceRestriction samlAudienceRestriction = it.next();
                    if (samlAudienceRestriction.getAudiences() != null) {
                        Iterator<Audience> it2 = samlAudienceRestriction.getAudiences().iterator();
                        while (it2.hasNext()) {
                            Audience audience = it2.next();
                            audienceRestrictionList.add(audience.getAudienceURI());
                        }
                    }
                }
                conditions.setAudiencieRestriction(audienceRestrictionList);
            }
        }
        return conditions;
    }
}
