/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator.commons;

import eu.semiramis.commons.SAMLUtil;
import java.util.Date;
import java.util.List;
import org.opensaml.saml2.core.Action;
import org.opensaml.saml2.core.Assertion;
import org.opensaml.saml2.core.Audience;
import org.opensaml.saml2.core.AudienceRestriction;
import org.opensaml.saml2.core.AuthzDecisionStatement;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.XMLParserException;

/**
 * Authorization Token
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class AuthorizationToken extends Token {

    public AuthorizationToken(Assertion assertion) throws XMLParserException, UnmarshallingException {
        Assertion assertionCopy = SAMLUtil.cloneAssertion(assertion);
        setAssertion(assertionCopy);
    }

    public AuthorizationToken(String assertion) throws XMLParserException, UnmarshallingException {
        setAssertion((Assertion) SAMLUtil.stringToSAML(assertion));
    }

    public String getAction() {
        String action = null;
        try {
            AuthzDecisionStatement authzDecisionStatement = assertion.getAuthzDecisionStatements().get(0);
            Action act = authzDecisionStatement.getActions().get(0);
            action = act.getAction();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return action;
    }

    public String getResource() {
        String resource = null;
        try {
            AuthzDecisionStatement authzDecisionStatement = assertion.getAuthzDecisionStatements().get(0);
            resource = authzDecisionStatement.getResource();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return resource;
    }

    public Date getNotOnOrAfter() {
        return assertion.getConditions().getNotOnOrAfter().toDate();
    }

    public String getAudienceURI() {
        List<AudienceRestriction> audienceRestrictions = assertion.getConditions().getAudienceRestrictions();
        if (audienceRestrictions.isEmpty()) {
            return null;
        }
        List<Audience> audiences = audienceRestrictions.get(0).getAudiences();
        if (audiences.isEmpty()) {
            return null;
        }
        return audiences.get(0).getAudienceURI();
    }
}
