/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator.commons;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Definition of Provider
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class Provider {

    private String id;
    private String url;
    private String certPath;
    private boolean signMessages = true;
    private boolean checkSignatures = true;
    private boolean encryptAssertions = false;
    private boolean signOutcomingTokens = false;
    private boolean checkAssertionTime = true;
    private boolean usePseudonyms = false;
    private boolean generateNewTokens = false;

    public Provider() {
    }

    public Provider(Node item, String BASEPATH) {
        id = item.getAttributes().getNamedItem("id").getNodeValue();
        NodeList nodeList = item.getChildNodes();

        for (int i = 0; i < nodeList.getLength(); i++) {

            Node childNode = nodeList.item(i);
            String name = childNode.getNodeName();
            if (name.equals("CertPath")) {
                certPath = childNode.getTextContent();
                if (!certPath.startsWith("/")) {
                    certPath = BASEPATH + certPath;
                }
            } else if (name.equals("URL")) {
                url = childNode.getTextContent();
            } else if (name.equals("SignMessages")) {
                signMessages = Boolean.parseBoolean(childNode.getTextContent());
            } else if (name.equals("CheckSignatures")) {
                checkSignatures = Boolean.parseBoolean(childNode.getTextContent());
            } else if (name.equals("EncryptAssertions")) {
                encryptAssertions = Boolean.parseBoolean(childNode.getTextContent());
            } else if (name.equals("SignOutcomingTokens")) {
                signOutcomingTokens = Boolean.parseBoolean(childNode.getTextContent());
            } else if (name.equals("CheckAssertionTime")) {
                checkAssertionTime = Boolean.parseBoolean(childNode.getTextContent());
            } else if (name.equals("GenerateNewTokens")) {
                generateNewTokens = Boolean.parseBoolean(childNode.getTextContent());
            } else if (name.equals("UsePseudonyms")) {
                usePseudonyms = Boolean.parseBoolean(childNode.getTextContent());
            }
        }
    }

    public boolean isCheckAssertionTime() {
        return checkAssertionTime;
    }

    public void setCheckAssertionTime(boolean checkAssertionTime) {
        this.checkAssertionTime = checkAssertionTime;
    }

    public boolean isCheckSignatures() {
        return checkSignatures;
    }

    public void setCheckSignatures(boolean checkSignatures) {
        this.checkSignatures = checkSignatures;
    }

    public boolean isEncryptAssertions() {
        return encryptAssertions;
    }

    public void setEncryptAssertions(boolean encryptAssertions) {
        this.encryptAssertions = encryptAssertions;
    }

    public boolean isSignMessages() {
        return signMessages;
    }

    public void setSignMessages(boolean signMessages) {
        this.signMessages = signMessages;
    }

    public boolean isSignOutcomingTokens() {
        return signOutcomingTokens;
    }

    public void setSignOutcomingTokens(boolean signOutcomingTokens) {
        this.signOutcomingTokens = signOutcomingTokens;
    }

    public boolean isGenerateNewTokens() {
        return generateNewTokens;
    }

    public void setGenerateNewTokens(boolean generateNewTokens) {
        this.generateNewTokens = generateNewTokens;
    }

    public boolean isUsePseudonyms() {
        return usePseudonyms;
    }

    public void setUsePseudonyms(boolean usePseudonyms) {
        this.usePseudonyms = usePseudonyms;
    }

    public String getCertPath() {
        return certPath;
    }

    public void setCertPath(String certPath) {
        this.certPath = certPath;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getUrl() {
        return url;
    }

    public void setUrl(String url) {
        this.url = url;
    }
}
