/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.commons.audit.Transaction;
import eu.semiramis.identityaggregator.Audit.EventId;
import eu.semiramis.identityaggregator.Audit.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * Identity Aggregator - Local Audit Module
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class AuditLocal implements AuditDelegate {

    private Logger logger;

    public AuditLocal(String handlerName, String logConfigPath) {
        PropertyConfigurator.configure(logConfigPath);
        logger = Logger.getLogger(handlerName);
    }

    @Override
    public void audit(Level level, Transaction transaction, EventId eventId, String msg) {
        switch (level) {
            case DEBUG:
                logger.debug(eventId.getEventId() + ": " + msg);
                break;
            case ERROR:
                logger.error(eventId.getEventId() + ": " + msg);
                break;
            case INFO:
                logger.info(eventId.getEventId() + ": " + msg);
                break;
            case TRACE:
                logger.trace(eventId.getEventId() + ": " + msg);
                break;
            case WARN:
                logger.warn(eventId.getEventId() + ": " + msg);
                break;
        }
    }

    @Override
    public void audit(Level level, Transaction transaction, EventId eventId, Exception ex) {
        switch (level) {
            case DEBUG:
                logger.debug(eventId.getEventId() + ": " + ex.getMessage(), ex);
                break;
            case ERROR:
                logger.error(eventId.getEventId() + ": " + ex.getMessage(), ex);
                break;
            case INFO:
                logger.info(eventId.getEventId() + ": " + ex.getMessage(), ex);
                break;
            case TRACE:
                logger.trace(eventId.getEventId() + ": " + ex.getMessage(), ex);
                break;
            case WARN:
                logger.warn(eventId.getEventId() + ": " + ex.getMessage(), ex);
                break;
        }
    }
}
