/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.commons.audit.AuditFactory;
import eu.semiramis.commons.audit.Transaction;
import eu.semiramis.identityaggregator.Audit.EventId;
import eu.semiramis.identityaggregator.Audit.Level;

/**
 * Identity Aggregator - Audit Module
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class AuditServer implements AuditDelegate {

    private eu.semiramis.commons.audit.Audit audit;

    public AuditServer(String handlerName) {
        this.audit = AuditFactory.getAudit(handlerName);
    }

    public void audit(Level level, Transaction transaction, EventId eventId, String msg) {
        audit.audit(translateLevel(level), transaction, eventId.getEventId(), msg);
    }

    public void audit(Level level, Transaction transaction, EventId eventId, Exception ex) {
        audit.audit(translateLevel(level), transaction, eventId.getEventId(), ex);
    }

    private eu.semiramis.commons.audit.Audit.Level translateLevel(Level level) {
        eu.semiramis.commons.audit.Audit.Level newLevel = null;
        switch (level) {
            case DEBUG:
                newLevel = eu.semiramis.commons.audit.Audit.Level.DEBUG;
                break;
            case ERROR:
                newLevel = eu.semiramis.commons.audit.Audit.Level.ERROR;
                break;
            case INFO:
                newLevel = eu.semiramis.commons.audit.Audit.Level.INFO;
                break;
            case TRACE:
                newLevel = eu.semiramis.commons.audit.Audit.Level.TRACE;
                break;
            case WARN:
                newLevel = eu.semiramis.commons.audit.Audit.Level.WARN;
                break;
        }
        return newLevel;
    }
}
