/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.identityaggregator.Audit.Level;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Identity Aggregator Server Component - WAYF Service
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class IAWAYF {

    protected IdentityAggregator_Config config;
    protected Audit audit;

    public IAWAYF(String configFilePath) throws Exception {
        config = new IdentityAggregator_Config(configFilePath);
        init();
    }

    protected void init() {
        try {
            audit = new Audit(IAWAYF.class.getName());
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void processAndRedirect(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        String origin = request.getParameter("origin");
        String relayState = request.getParameter("RelayState");
        String samlRequest = request.getParameter("SAMLRequest");
        String destination = request.getParameter("destination");
        audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "origin=" + origin);
        audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "RelayState=" + relayState);
        audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "samlRequest=" + samlRequest);
        audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "destination=" + destination);
        if (origin != null) {
            try {
                if (config.WAYF_SAVE_ORIGIN_IN_SESSION) {
                    // Store IA Origin in session to know in future interactions.
                    request.getSession(true).setAttribute("origin", origin);
                }

                Map<String, String> parameters = new HashMap<String, String>();
                parameters.put("Origin", origin);
                parameters.put("RelayState", relayState);
                parameters.put("SAMLRequest", samlRequest);
                post(destination, parameters, response);


            } catch (Exception ex) {
                ex.printStackTrace();
                audit.audit(Level.ERROR, request, Audit.AppLogEventId.WAYF_ERROR, ex);
            }
        }
    }

    private void post(String url, Map<String, String> parameters, HttpServletResponse response) throws IOException {
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter out = response.getWriter();
        out.println("<html>");
        out.println("<body  onload=\"document.forms[0].submit()\">");
        out.println("<noscript>");
        out.println("<p>");
        out.println("<strong>Note:");
        out.println("</strong> Since your browser does not support JavaScript, you must press the Continue button once to proceed.");
        out.println("</p>");
        out.println("</noscript>");
        out.println("<form method=\"post\" action=" + url + ">");
        Iterator<String> it = parameters.keySet().iterator();
        while (it.hasNext()) {
            String paramName = it.next();
            String paramValue = parameters.get(paramName);
            out.println("<input type=\"hidden\" name=\"" + paramName + "\" value=\"" + paramValue + "\"/>");
        }
        out.println("<noscript>");
        out.println("<input type=\"submit\" value=\"Continue\"><p>");
        out.println("</noscript>");
        out.println("</form>");
        out.println("</body>");
        out.println("</html>");
        out.close();
    }
}
