/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.identityaggregator.commons.Provider;
import eu.semiramis.identityaggregator.Audit.Level;
import eu.semiramis.identityaggregator.commons.Attribute;
import eu.semiramis.identityaggregator.connector.IAConnector;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import javax.servlet.http.HttpSession;
import org.opensaml.saml2.core.AuthnRequest;
import org.opensaml.saml2.core.RequestAbstractType;
import org.opensaml.saml2.core.SubjectQuery;
import org.opensaml.xml.util.Base64;
import org.opensaml.xml.util.XMLHelper;

/**
 * Identity Aggregator - Id & Discovery Module
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class ID_Discovery {

    private IdentityAggregator_Config config;
    private Audit audit;
    private IAConnector iaConnector = null;

    ID_Discovery(IdentityAggregator_Config config) {
        this.config = config;
        audit = new Audit(ID_Discovery.class.getName());
    }

    private String getWAYF() throws IOException {
        if (config.WAYF_URL != null) {
            return config.WAYF_URL;
        } else {
            throw new IOException("No WAYF Service configurated");
        }
    }

    public void showWAYF(HttpServletRequest request, HttpServletResponse response, RequestMessage requestMessage) throws IOException {
        // check if it must use internal WAYF instead of being redirected
        if (config.USE_INTERNAL_WAYF) {
            audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Showing internal WAYF");
            InternalWAYF(request, response, requestMessage);
        } else {
            // use an external WAYF Service
            String url = getWAYF();
            response.sendRedirect(url);
        }
    }

    private void InternalWAYF(HttpServletRequest request, HttpServletResponse response, RequestMessage requestMessage) throws IOException {
        String encodedMessage = "";
        String destination = "";
        if (requestMessage instanceof SAMLRequestMessage) {
            audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "Encoding SAMLmessage to Base64...");
            String messageXML = XMLHelper.nodeToString(((SAMLRequestMessage) requestMessage).getSamlRequest().getDOM());
            encodedMessage = Base64.encodeBytes(messageXML.getBytes(), Base64.DONT_BREAK_LINES);
            audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "Done.");
            destination = ((SAMLRequestMessage) requestMessage).getSamlRequest().getDestination();
        }

        String requestedService = getRequestedService(requestMessage);

        List<WAYFDestination> wayfDestinations;
        if (config.WAYF_internalRelyingParties.contains(requestMessage.getIssuer())) {
            wayfDestinations = config.WAYF_outgoingDestinations.get(requestedService);
        } else {
            wayfDestinations = config.WAYF_incomingDestinations.get(requestedService);
        }

        audit.audit(Level.DEBUG, request, Audit.AppLogEventId.WAYF_INFO, "Showing WAYF web page to user...");
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter out = response.getWriter();
        try {
            out.println("<html>");
            out.println("<head>");
            out.println("<script type=\"text/javascript\">");
            out.println("function openWindowWithPost(url,name,keys,values)");
            out.println("{");
            out.println("window.open(url,'_top');");
            out.println("var html = \"\";");
            out.println("html += \"<html><head></head><body><form id='formid' method='post' action='\" + url +\"'>\";");
            out.println("if (keys && values && (keys.length == values.length))");
            out.println("for (var i=0; i < keys.length; i++)");
            out.println("html += \"<input type='hidden' name='\" + keys[i] + \"' value='\" + values[i] + \"'/>\";");
            out.println("html += \"</form><script type='text/javascript'>document.getElementById(\\\"formid\\\").submit()</sc\"+\"ript></body></html>\";");
            out.println("window.document.write(html);");
            out.println("return window;");
            out.println("}");
            out.println("function clickButton(origin)");
            out.println("{");
            out.println("keys=new Array(\"origin\",\"RelayState\",\"SAMLRequest\",\"destination\");");
            out.println("values=new Array(origin,\"" + requestMessage.getConsumerServiceURL() + "\",\"" + encodedMessage + "\",\"" + destination + "\");");
            out.println("openWindowWithPost('" + config.WAYF_URL + "','',keys,values);");
            out.println("}");
            out.println("</script>");
            out.println("<title>WAYF</title>");
            out.println("</head>");
            out.println("<body>");
            out.println("<p>Type Where Are You From</p>");
            out.println("<form action=\"" + "../" + "\" >");
            out.println("<select id=\"myOrigin\">");
            Iterator<WAYFDestination> it = wayfDestinations.iterator();
            while (it.hasNext()) {
                WAYFDestination wayfDestination = it.next();
                out.println("<option value=\"" + wayfDestination.getId() + "\">" + wayfDestination.getName() + "</option>");
            }
            out.println("</select>");
            out.println("<input type=\"button\" value=\"Go there\"");
            out.println("onClick=\"ob=this.form.myOrigin;clickButton(ob.options[ob.selectedIndex].value)\"");
            out.println("/>");

            out.println("</form>");

            out.println("</body>");
            out.println("</html>");
        } finally {
            out.close();
        }
    }

    public Destination discoverAuthnProviderURL(HttpServletRequest request, RequestMessage requestMessage) {
        return discoverProviderURL(request, requestMessage, config.SEMIRAMIS_AUTHP_URL);
    }

    public Map<Destination, List<Attribute>> discoverAttrProviderURL(HttpServletRequest request, RequestMessage requestMessage) {
        Map<Destination, List<Attribute>> providers = new HashMap<Destination, List<Attribute>>();
        Destination destination = discoverProviderURL(request, requestMessage, config.SEMIRAMIS_ATTRP_URL);
        if (destination != null) {
            providers.put(destination, requestMessage.getRequestedAttributes());
        } else {
            Map<String, Destination> aux = new HashMap<String, Destination>();
            Iterator<Attribute> it = requestMessage.getRequestedAttributes().iterator();
            while (it.hasNext()) {
                Attribute attribute = it.next();
                String domainId = getDomain(request, attribute.getId(), requestMessage.getSubjectId());
                if (domainId != null) {
                    if (!aux.containsKey(domainId)) {
                        destination = getDestinationFromId(request, domainId);
                        aux.put(domainId, destination);
                    }
                    destination = aux.get(domainId);
                    if (!providers.containsKey(destination)) {
                        providers.put(destination, new ArrayList<Attribute>());
                    }
                    providers.get(destination).add(attribute);
                }
            }
        }
        return providers;
    }

    public Destination discoverAuthzProviderURL(HttpServletRequest request, RequestMessage requestMessage) {
        return discoverProviderURL(request, requestMessage, config.SEMIRAMIS_AUTHZ_URL);
    }

    private Destination discoverProviderURL(HttpServletRequest request, RequestMessage requestMessage, String defaultProvider) {
        // checks if origin is specified in the request message
        audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Checking if user origin is specified in the request message...");
        Destination destination = searchOriginInRequestMessage(request, requestMessage);
        if (destination != null) {
            audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Origin found: " + destination.getProviderId());
            return destination;
        }
        // check if the origin is specified in the HTTP parameters (e.g. user comes from the WAYF service)
        audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Checking if origin is specified in the HTTP parameters...");
        destination = searchOriginInRequestParameters(request, requestMessage);
        if (destination != null) {
            audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Origin found: " + destination.getProviderId());
            return destination;
        }
        // check if the user has previously selected his/her origin
        audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Checking if the user has previously selected his/her origin...");
        destination = searchOriginInSession(request, requestMessage);
        if (destination != null) {
            audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Origin found: " + destination.getProviderId());
            return destination;
        }
        // by default return the Provider configured
        audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Checking if user origin is configured by default...");
        if (defaultProvider != null) {
            audit.audit(Level.INFO, request, Audit.AppLogEventId.DISCOVERY_INFO, "Origin found: " + defaultProvider);
            destination = new Destination();
            destination.setProviderId(defaultProvider);
            destination.setUrl(getProviderURL(request, defaultProvider));
            destination.setDomain(null);
            return destination;
        }
        audit.audit(Level.INFO, request, Audit.AppLogEventId.USER_ORIGIN_NOT_FOUND, "User origin has not been found");
        return null;
    }

    private String getRequestedService(RequestMessage requestMessage) {
        RequestType requestType = requestMessage.getType();
        String requestedService = null;
        if (requestType.equals(RequestType.AUTHENTICATION)) {
            requestedService = "Authentication";
        } else if (requestType.equals(RequestType.AUTHORIZATION)) {
            requestedService = "Authorization";
        } else if (requestType.equals(RequestType.ATTRIBUTE)) {
            requestedService = "Attributes";
        } else {
            throw new IllegalArgumentException("Invalid request type: " + requestType);
        }
        return requestedService;
    }

    private Destination getDestinationFromId(HttpServletRequest request, String destinationId) {
        if (!config.WAYF_route.containsKey(destinationId)) {
            audit.audit(Level.ERROR, request, Audit.AppLogEventId.DESTINATION_NOT_FOUND, "Destination not found in WAYF config: " + destinationId);
            return null;
        }
        Destination destination = new Destination();
        destination.setDomain(destinationId);
        destination.setProviderId(getRoute(destinationId));
        destination.setUrl(getProviderURL(request, destination.getProviderId()));
        return destination;
    }

    private Destination searchOriginInSession(HttpServletRequest request, RequestMessage requestMessage) {
        try {
            HttpSession session = request.getSession();
            if (session != null) {
                String origin = (String) session.getAttribute("origin");
                if (origin != null) {
                    String requestedService = getRequestedService(requestMessage);
                    if (config.WAYF_route.get(origin).getServices().contains(requestedService)) {
                        return getDestinationFromId(request, origin);
                    }
                }
            }
            return null;
        } catch (Throwable ex) {
            return null;
        }
    }

    private Destination searchOriginInRequestParameters(HttpServletRequest request, RequestMessage requestMessage) {
        try {
            String origin = (String) request.getParameter("Origin");
            if (origin != null) {
                String requestedService = getRequestedService(requestMessage);
                if (config.WAYF_route.get(origin).getServices().contains(requestedService)) {
                    return getDestinationFromId(request, origin);
                }
            }
            return null;
        } catch (Throwable ex) {
            return null;
        }
    }

    private Destination searchOriginInRequestMessage(HttpServletRequest request, RequestMessage requestMessage) {
        String destinationId = null;
        if (requestMessage instanceof SAMLRequestMessage) {
            RequestAbstractType samlRequest = ((SAMLRequestMessage) requestMessage).getSamlRequest();
            String type = samlRequest.getElementQName().getLocalPart();
            try {
                if (type.equals("AuthnRequest")) {
                    AuthnRequest authnRequest = (AuthnRequest) samlRequest;
                    destinationId = authnRequest.getScoping().getIDPList().getIDPEntrys().get(0).getProviderID();
                } else if (type.equals("AttributeQuery") || type.equals("AuthzDecisionQuery")) {
                    SubjectQuery subjectQuery = (SubjectQuery) samlRequest;
                    destinationId = subjectQuery.getSubject().getNameID().getNameQualifier();
                }
            } catch (Exception ex) {
                audit.audit(Level.INFO, request, Audit.AppLogEventId.DESTINATION_NOT_FOUND, "No destination info found in the SAML message");
            }
        }
        if (destinationId != null) {
            return getDestinationFromId(request, destinationId);
        }
        return null;
    }

    /*   public String getProviderURL(String providerId) {
    audit.audit(Level.DEBUG, "Searching URL for id:"+providerId);
    Iterator<Provider> it = config.Providers.iterator();
    while (it.hasNext()) {
    Provider provider = it.next();
    if (provider.getId().equals(providerId)){
    audit.audit(Level.DEBUG, "URL found:"+provider.getUrl());
    return provider.getUrl();
    }
    }
    return null;
    }
     */
    public String getProviderURL(HttpServletRequest request, String providerId) {
        audit.audit(Level.DEBUG, request, Audit.AppLogEventId.DISCOVERY_INFO, "Searching URL for id:" + providerId);
        if (config.Providers.containsKey(providerId)) {
            Provider provider = config.Providers.get(providerId);
            audit.audit(Level.DEBUG, request, Audit.AppLogEventId.DISCOVERY_INFO, "URL found:" + provider.getUrl());
            return provider.getUrl();
        }
        return null;
    }

    public String getNotificationEmailAddress(HttpServletRequest request, String subjectId) {
        return getIAConnector(request).getNotificationEmailAddress(subjectId);
    }

    private String getRoute(String destinationId) {
        WAYFDestination wayfDestination = config.WAYF_route.get(destinationId);
        return wayfDestination.getRoute();
    }

    private String getDomain(HttpServletRequest request, String attributeId, String subjectId) {
        return getIAConnector(request).getDomain(attributeId, subjectId);
    }

    private IAConnector getIAConnector(HttpServletRequest request) {
        if (this.iaConnector == null) {
            try {
                Class clazzPKI = Class.forName(config.IACONNECTOR_IMPL_CLASS);
                Constructor constructorPKI = clazzPKI.getConstructor(String.class);
                this.iaConnector = (IAConnector) constructorPKI.newInstance(config.IACONNECTOR_CONFIG_FILE_PATH);
            } catch (Exception ex) {
                audit.audit(Level.ERROR, request, Audit.AppLogEventId.IA_CONNECTOR_INSTANTIATION_ERROR, ex);
            }
        }
        return this.iaConnector;
    }
    /*public String urlToId(String url){
    audit.audit(Level.DEBUG, "Converting URL to ID. URL="+url);
    Iterator<String> it =  config.Providers.keySet().iterator();
    while (it.hasNext()) {
    String id = it.next();
    if(config.Providers.get(id).equals(url)){
    audit.audit(Level.DEBUG, "Id found="+id);
    return id;
    }
    }
    audit.audit(Level.DEBUG, "Id has not be found");
    return null;
    }*/
}
