/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.identityaggregator.commons.Provider;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.AbstractMap.SimpleEntry;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import org.opensaml.xml.parse.BasicParserPool;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Identity Aggregator - Configuration class
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class IdentityAggregator_Config {

    private Properties properties = null;

    public IdentityAggregator_Config(String configFilePath) throws Exception {
        loadConfigFile(configFilePath);
    }

    protected IdentityAggregator_Config() {
    }

    protected void loadConfigFile(String configFilePath) throws Exception {
        properties = new Properties();
        try {
            properties.loadFromXML(new FileInputStream(configFilePath));
            Enumeration<Object> keys = properties.keys();
            for (; keys.hasMoreElements();) {
                String actual = (String) keys.nextElement();
                setProperty(actual, properties.getProperty(actual));
            }
        } catch (IOException ex) {
            Logger.getLogger(IdentityAggregator_Config.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    protected void setProperty(String current, String property) throws Exception {
        if (current.equals("BASEPATH")) {
            BASEPATH = property;
        }

        if (current.contains("_PATH")) {
            if (!property.startsWith("/")) {
                property = properties.getProperty("BASEPATH") + property;
            }
        }
        if (current.equals("ID")) {
            ID = property;
        }
        if (current.equals("ORGANIZATION")) {
            ORGANIZATION = property;
        }
        if (current.equals("CONTACT_INFO")) {
            CONTACT_INFO = property;
        }
        if (current.equals("DOMAIN")) {
            DOMAIN = property;
        }
        if (current.equals("SEMIRAMIS_AUTHP_URL")) {
            SEMIRAMIS_AUTHP_URL = property;
        }
        if (current.equals("SEMIRAMIS_ATTRP_URL")) {
            SEMIRAMIS_ATTRP_URL = property;
        }
        if (current.equals("SEMIRAMIS_AUTHZ_URL")) {
            SEMIRAMIS_AUTHZ_URL = property;
        }
        if (current.equals("LOG_CONFIG_PATH")) {
            LOG_CONFIG_PATH = property;
        }
        if (current.equals("WAYF_URL")) {
            WAYF_URL = property;
        }
        if (current.equals("AUTO_AUTHENTICATION")) {
            AUTO_AUTHENTICATION = Boolean.parseBoolean(property);
        }
        if (current.equals("USE_INTERNAL_WAYF")) {
            USE_INTERNAL_WAYF = Boolean.parseBoolean(property);
        }
        if (current.equals("API_PKCS12_PATH")) {
            API_PKCS12_PATH = property;
        }
        if (current.equals("API_PKCS12_PASSWORD")) {
            API_PKCS12_PASSWORD = property;
        }
        if (current.equals("API_PKCS12_CERT_ALIAS")) {
            API_PKCS12_CERT_ALIAS = property;
        }
        if (current.equals("API_PRIVATE_KEY_PASSWORD")) {
            API_PRIVATE_KEY_PASSWORD = property;
        }
        if (current.equals("IACONNECTOR_IMPL_CLASS")) {
            IACONNECTOR_IMPL_CLASS = property;
        }
        if (current.equals("IACONNECTOR_CONFIG_FILE_PATH")) {
            IACONNECTOR_CONFIG_FILE_PATH = property;
        }
        if (current.equals("RELYING_PARTY_FILE_PATH")) {
            RELYING_PARTY_FILE_PATH = property;
            //read SP config file
            BasicParserPool b = new BasicParserPool();
            FileInputStream f = new FileInputStream(property);
            Document d = b.parse(f);

            NodeList nodeList = d.getElementsByTagName("RP");
            for (int i = 0; i < nodeList.getLength(); i++) {
                Provider provider = new Provider(nodeList.item(i), properties.getProperty("BASEPATH"));
                Providers.put(provider.getId(), provider);
            }
        }
        if (current.equals("ENABLE_AUDIT_SYSTEM")) {
            ENABLE_AUDIT_SYSTEM = Boolean.parseBoolean(property);
        }
        if (current.equals("DISABLE_MSG_CORRELATION_AUDIT")) {
            DISABLE_MSG_CORRELATION_AUDIT = Boolean.parseBoolean(property);
        }
        if (current.equals("ENABLE_LOG_TO_AUDIT_SYSTEM")) {
            ENABLE_LOG_TO_AUDIT_SYSTEM = Boolean.parseBoolean(property);
        }
        if (current.equals("AUDIT_SERVER_CONFIG_PATH")) {
            AUDIT_SERVER_CONFIG_PATH = property;
        }
        if (current.equals("USER_CONSENT_DISPATCHER")) {
            USER_CONSENT_DISPATCHER = property;
        }
        if (current.equals("POLICY_MANAGER")) {
            POLICY_MANAGER = property;
        }
        if (current.equals("POLICY_MGR_CONFIG_FILE_PATH")) {
            POLICY_MGR_CONFIG_FILE_PATH = property;
        }
        if (current.equals("ENABLE_TRUST_POLICIES")) {
            ENABLE_TRUST_POLICIES = Boolean.parseBoolean(property);
        }
        if (current.equals("MAIL_SMTP_HOST")) {
            MAIL_SMTP_HOST = property;
        }
        if (current.equals("MAIL_SMTP_PORT")) {
            MAIL_SMTP_PORT = property;
        }
        if (current.equals("MAIL_USE_SSL")) {
            MAIL_USE_SSL = Boolean.parseBoolean(property);
        }
        if (current.equals("MAIL_AUTHENTICATION_USER")) {
            MAIL_AUTHENTICATION_USER = property;
        }
        if (current.equals("MAIL_AUTHENTICATION_PASSWORD")) {
            MAIL_AUTHENTICATION_PASSWORD = property;
        }
        if (current.equals("MAIL_FROM_ADDRESS")) {
            MAIL_FROM_ADDRESS = property;
        }
        if (current.equals("NOTIFICATION_MAIL_SUBJECT")) {
            NOTIFICATION_MAIL_SUBJECT = property;
        }
        if (current.equals("NOTIFICATION_MAIL_TEMPLATE_PATH")) {
            NOTIFICATION_MAIL_TEMPLATE_PATH = property;
        }
        if (current.equals("ATTRIBUTE_TRANSLATOR_FILE_PATH")) {
            ATTRIBUTE_TRANSLATOR_FILE_PATH = property;

            BasicParserPool b = new BasicParserPool();
            FileInputStream f = new FileInputStream(property);
            Document d = b.parse(f);

            NodeList nodeList = d.getElementsByTagName("ATTRIBUTE_DICTIONARY");
            for (int i = 0; i < nodeList.getLength(); i++) {
                String origin = nodeList.item(i).getAttributes().getNamedItem("origin").getNodeValue();
                String target = nodeList.item(i).getAttributes().getNamedItem("target").getNodeValue();
                Entry<String, String> entry = new SimpleEntry<String, String>(origin, target);
                HashMap<String, String> dictionary = new HashMap<String, String>();
                Entry<String, String> revEntry = new SimpleEntry<String, String>(target, origin);
                HashMap<String, String> revDictionary = new HashMap<String, String>();
                AttributeDictionariesMap.put(entry, dictionary);
                AttributeDictionariesMap.put(revEntry, revDictionary);

                NodeList attributeNodeList = nodeList.item(i).getOwnerDocument().getElementsByTagName("ATTRIBUTE");
                for (int j = 0; j < attributeNodeList.getLength(); j++) {
                    String originId = attributeNodeList.item(j).getAttributes().getNamedItem("originAttr").getNodeValue();
                    String targetId = attributeNodeList.item(j).getAttributes().getNamedItem("targetAttr").getNodeValue();
                    dictionary.put(originId, targetId);
                    revDictionary.put(targetId, originId);
                }

            }

            nodeList = d.getElementsByTagName("ATTRIBUTE_GROUP");
            for (int i = 0; i < nodeList.getLength(); i++) {
                String groupId = nodeList.item(i).getAttributes().getNamedItem("id").getNodeValue();

                NodeList attributeNodeList = nodeList.item(i).getOwnerDocument().getElementsByTagName("ATTRIBUTE_NAME");
                for (int j = 0; j < attributeNodeList.getLength(); j++) {
                    String attributeId = attributeNodeList.item(j).getAttributes().getNamedItem("id").getNodeValue();
                    if (!AttributeGroupMap.containsKey(attributeId)) {
                        AttributeGroupMap.put(attributeId, new ArrayList<String>());
                    }
                    List<String> groupList = AttributeGroupMap.get(attributeId);
                    groupList.add(groupId);
                }
            }
        }


        if (current.equals("IDENTITY_TRANSLATOR_FILE_PATH")) {
            IDENTITY_TRANSLATOR_FILE_PATH = property;

            BasicParserPool b = new BasicParserPool();
            FileInputStream f = new FileInputStream(property);
            Document d = b.parse(f);

            NodeList nodeList = d.getElementsByTagName("IDENTITY_DICTIONARY");
            for (int i = 0; i < nodeList.getLength(); i++) {
                String origin = nodeList.item(i).getAttributes().getNamedItem("origin").getNodeValue();
                String target = nodeList.item(i).getAttributes().getNamedItem("target").getNodeValue();
                Entry<String, String> entry = new SimpleEntry<String, String>(origin, target);
                HashMap<String, String> dictionary = new HashMap<String, String>();
                Entry<String, String> revEntry = new SimpleEntry<String, String>(target, origin);
                HashMap<String, String> revDictionary = new HashMap<String, String>();
                IdentityDictionariesMap.put(entry, dictionary);
                IdentityDictionariesMap.put(revEntry, revDictionary);

                NodeList attributeNodeList = nodeList.item(i).getOwnerDocument().getElementsByTagName("IDENTITY");
                for (int j = 0; j < attributeNodeList.getLength(); j++) {
                    String originId = attributeNodeList.item(j).getAttributes().getNamedItem("originID").getNodeValue();
                    String targetId = attributeNodeList.item(j).getAttributes().getNamedItem("targetID").getNodeValue();
                    dictionary.put(originId, targetId);
                    revDictionary.put(targetId, originId);
                }

            }
        }

        if (current.equals("WAYF_CONFIG_FILE_PATH")) {
            WAYF_CONFIG_FILE_PATH = property;
            //read WAYF config file
            BasicParserPool b = new BasicParserPool();
            FileInputStream f = new FileInputStream(property);
            Document d = b.parse(f);

            XPath xPath = XPathFactory.newInstance().newXPath();
            WAYF_URL = xPath.evaluate("/wayfConfig/url", d);

            WAYF_SAVE_ORIGIN_IN_SESSION = Boolean.parseBoolean(xPath.evaluate("/wayfConfig/saveOriginInSession", d));

            NodeList outgoingDestinations = (NodeList) xPath.evaluate("/wayfConfig/outgoingDestinations/destination", d, XPathConstants.NODESET);
            addWAYFDestinations(WAYF_outgoingDestinations, outgoingDestinations);

            NodeList incomingDestinations = (NodeList) xPath.evaluate("/wayfConfig/incomingDestinations/destination", d, XPathConstants.NODESET);
            addWAYFDestinations(WAYF_incomingDestinations, incomingDestinations);

            NodeList internalRelyingParties = (NodeList) xPath.evaluate("/wayfConfig/internalRelyingParties/internalRP", d, XPathConstants.NODESET);
            for (int i = 0; i < internalRelyingParties.getLength(); i++) {
                WAYF_internalRelyingParties.add(internalRelyingParties.item(i).getTextContent());
            }
        }
    }

    /**
     * Adds WAYF destinations to WAYF_outgoingDestinations and WAYF_incomingDestinations maps.
     * Destinations are classified by service in these maps, i.e., each service is associated with a list of destinations providing that service.
     */
    private void addWAYFDestinations(Map<String, List<WAYFDestination>> wayfDestinations, NodeList domDestinations) throws XPathExpressionException {
        XPath xPath = XPathFactory.newInstance().newXPath();
        for (int i = 0; i < domDestinations.getLength(); i++) {
            Node domDestination = domDestinations.item(i);
            WAYFDestination destination = new WAYFDestination();
            String destinationId = xPath.evaluate("@id", domDestination);
            destination.setId(destinationId);
            destination.setName(xPath.evaluate("name", domDestination));
            destination.setRoute(xPath.evaluate("route", domDestination));

            NodeList services = (NodeList) xPath.evaluate("services/service", domDestination, XPathConstants.NODESET);
            for (int j = 0; j < services.getLength(); j++) {
                String service = services.item(j).getTextContent();
                destination.addService(service);
                List<WAYFDestination> serviceDestinations = wayfDestinations.get(service);
                if (serviceDestinations == null) {
                    serviceDestinations = new ArrayList<WAYFDestination>();
                    wayfDestinations.put(service, serviceDestinations);
                }
                serviceDestinations.add(destination);
            }
            WAYF_route.put(destinationId, destination);
        }
    }

    protected Properties getProperties() {
        return properties;
    }
    public String BASEPATH;
    public String ID;
    public String ORGANIZATION;
    public String CONTACT_INFO;
    public String DOMAIN;
    public String SEMIRAMIS_AUTHP_URL;
    public String SEMIRAMIS_ATTRP_URL;
    public String SEMIRAMIS_AUTHZ_URL;
    public boolean AUTO_AUTHENTICATION = false;
    public boolean USE_INTERNAL_WAYF = true;
    public String LOG_CONFIG_PATH;
    public String RELYING_PARTY_FILE_PATH;
    public String WAYF_CONFIG_FILE_PATH;
    public String WAYF_URL;
    public boolean WAYF_SAVE_ORIGIN_IN_SESSION;
    public String ATTRIBUTE_TRANSLATOR_FILE_PATH;
    public String IDENTITY_TRANSLATOR_FILE_PATH;
    public String API_PKCS12_PATH;
    public String API_PKCS12_PASSWORD;
    public String API_PKCS12_CERT_ALIAS;
    public String API_PRIVATE_KEY_PASSWORD;
    public String IACONNECTOR_IMPL_CLASS;
    public String IACONNECTOR_CONFIG_FILE_PATH;
    public boolean ENABLE_AUDIT_SYSTEM = true;
    public boolean DISABLE_MSG_CORRELATION_AUDIT = false;
    public boolean ENABLE_LOG_TO_AUDIT_SYSTEM = false;
    public String AUDIT_SERVER_CONFIG_PATH;
    public String USER_CONSENT_DISPATCHER;
    public String POLICY_MANAGER = PDPFactory.SEMIRAMIS_POLICY_MGR;    // Use SEMIRAMIS Policy Management System by default
    public String POLICY_MGR_CONFIG_FILE_PATH;
    public boolean ENABLE_TRUST_POLICIES;
    public String MAIL_SMTP_HOST;
    public String MAIL_SMTP_PORT;
    public boolean MAIL_USE_SSL;
    public String MAIL_AUTHENTICATION_USER;
    public String MAIL_AUTHENTICATION_PASSWORD;
    public String MAIL_FROM_ADDRESS;
    public String NOTIFICATION_MAIL_SUBJECT;
    public String NOTIFICATION_MAIL_TEMPLATE_PATH;
    public Map<String, WAYFDestination> WAYF_route = new HashMap<String, WAYFDestination>();
    public Map<String, List<WAYFDestination>> WAYF_outgoingDestinations = new HashMap<String, List<WAYFDestination>>();
    public Map<String, List<WAYFDestination>> WAYF_incomingDestinations = new HashMap<String, List<WAYFDestination>>();
    public List<String> WAYF_internalRelyingParties = new ArrayList<String>();
    public Map<String, Provider> Providers = new HashMap<String, Provider>();
    public HashMap<Entry<String, String>, HashMap<String, String>> AttributeDictionariesMap = new HashMap<Entry<String, String>, HashMap<String, String>>();
    public HashMap<String, List<String>> AttributeGroupMap = new HashMap<String, List<String>>();
    public HashMap<Entry<String, String>, HashMap<String, String>> IdentityDictionariesMap = new HashMap<Entry<String, String>, HashMap<String, String>>();
}
