/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import javax.servlet.http.HttpServletRequest;

/**
 * PDP
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public interface PDP {

    public enum Action {

        AUTHENTICATION,
        AUTHORIZATION,
        ATTRIBUTE_RELEASE;
    }

    /**
     * Evaluates the policies to determine whether the request is authorized or not
     * @param request the servlet request.
     * @param action the requested action
     * @param issuer the immediate issuer of the request
     * @param requestor the origin issuer of the request
     * @param target the target service that will serve the request
     * @param subject the id of the subject whose resource is requested
     * @param resource the resource that is being requested
     * @return a decision
     * @throws PDPException 
     */
    public Decision evaluate(HttpServletRequest request, Action action, String issuer, String requestor, String target, String subject, String resource) throws PDPException;
}
