/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.identityaggregator.commons.Token;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;

/**
 * Link outcoming SAMLRequest with incoming SAMLResponses
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class RequestLinker {

    static private HashMap<String, RequestMessage> requestMap = new HashMap<String, RequestMessage>();
    static private HashMap<RequestMessage, Map> providersMap = new HashMap<RequestMessage, Map>();
    static private HashMap<RequestMessage, List<Token>> tokensMap = new HashMap<RequestMessage, List<Token>>();
    static private HashMap<String, Map<HttpServletRequest, RequestMessage>> authenticationMap = new HashMap<String, Map<HttpServletRequest, RequestMessage>>();
    static private HashMap<String, List<Token>> authenticationTokensMap = new HashMap<String, List<Token>>();

    static public void linkRequest(String outcomingId, RequestMessage requestMessage) {
        requestMap.put(outcomingId, requestMessage);
    }

    static public RequestMessage getRequest(String inResponseTo) {
        return requestMap.get(inResponseTo);
    }

    static public void removeLink(String inResponseTo) {
        requestMap.remove(inResponseTo);
    }

    static public void linkProvidersMap(RequestMessage requestMessage, Map providers) {
        providersMap.put(requestMessage, providers);
    }

    static public Map getProvidersMap(RequestMessage requestMessage) {
        return providersMap.get(requestMessage);
    }

    static public void removeProvidersMap(RequestMessage requestMessage) {
        providersMap.remove(requestMessage);
    }

    static public void LinkResponseTokens(RequestMessage requestMessage, List<Token> tokens) {
        if (!tokensMap.containsKey(requestMessage)) {
            tokensMap.put(requestMessage, new ArrayList<Token>());
        }
        List<Token> storedTokens = tokensMap.get(requestMessage);
        storedTokens.addAll(tokens);
    }

    static public List<Token> getResponseTokens(RequestMessage requestMessage) {
        return tokensMap.get(requestMessage);
    }

    static public void removeResponseTokens(RequestMessage requestMessage) {
        tokensMap.remove(requestMessage);
    }

    static public void linkAuthenticationRequest(String authnReqId, Map<HttpServletRequest, RequestMessage> request) {
        authenticationMap.put(authnReqId, request);
    }

    static public Map<HttpServletRequest, RequestMessage> getAuthenticationLink(String authnReqId) {
        return authenticationMap.get(authnReqId);
    }

    static public void removeAuthenticationLink(String authnReqId) {
        authenticationMap.remove(authnReqId);
    }

    static public void linkAuthenticationTokens(String attrReqId, List<Token> authnTokens) {
        authenticationTokensMap.put(attrReqId, authnTokens);
    }

    static public List<Token> getAuthenticationTokens(String attrReqId) {
        return authenticationTokensMap.get(attrReqId);
    }

    static public void removeAuthenticationTokens(String attrReqId) {
        authenticationTokensMap.remove(attrReqId);
    }
}
