/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import eu.semiramis.identityaggregator.commons.Attribute;
import eu.semiramis.identityaggregator.commons.Conditions;
import eu.semiramis.identityaggregator.commons.Token;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;

/**
 * abstract RequestMessage
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public abstract class RequestMessage {

    public abstract String getId();

    public abstract RequestType getType();

    public abstract List<Token> getTokens();

    public abstract List<Attribute> getRequestedAttributes();

    public abstract String getConsumerServiceURL();

    public abstract String getIssuer();

    public abstract String getOriginIssuer();

    public abstract X509Certificate getAttrEncrX509Cert() throws CertificateException;

    public abstract String getSubjectId();

    public abstract void changeSubjectId(String subjectId);

    public abstract String getActionId();

    public abstract String getResourceId();

    public abstract Conditions getConditions();

    public abstract String getStatusCode();

    public abstract String getStatusMessage();

    /**
     * Methods kept for backward compatibility with (id, value) only attributes.
     */
    /**
     * @deprecated Replaced by getRequestedAttributes().
     */
    public abstract List<String> getAttributes();
}
