/*
 *    Copyright (C) 2012, 2013 Universidad de Murcia
 *
 *    Authors:
 *        Ginés Dólera Tormo <ginesdt@um.es>
 *        Juan M. Marín Pérez <juanmanuel@um.es>
 *        Jorge Bernal Bernabé <jorgebernal@um.es>
 *        Gregorio Martínez Pérez <gregorio@um.es>
 *        Antonio F. Skarmeta Gómez <skarmeta@um.es>
 *
 *    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).
 *
 *    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    SEMIRAMIS-IA is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
 *
 */
package eu.semiramis.identityaggregator;

import java.io.ByteArrayInputStream;
import java.io.Closeable;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.codec.binary.Base64;
import sun.security.x509.X500Name;

/**
 * IAServletDownloadCert
 *
 * @author Ginés Dólera Tormo
 * @author Juan M. Marín Pérez
 * @author Jorge Bernal Bernabé
 * @author Gregorio Martínez Pérez
 * @author Antonio F. Skarmeta Gómez
 */
public class IAServletDownloadCert extends HttpServlet {

    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        Closeable output = null;
        try {
            String certString = request.getParameter("certificate");
            if (certString == null) {
                response.setStatus(HttpServletResponse.SC_NOT_FOUND);
                response.setContentType("text/html;charset=UTF-8");
                PrintWriter writer = response.getWriter();
                output = writer;
                writer.println("<html>");
                writer.println("<head>");
                writer.println("<title>Download certificate</title>");
                writer.println("</head>");
                writer.println("<body>");
                writer.println("<h1>No certificate found in request.</h1>");
                writer.println("</body>");
                writer.println("</html>");
                writer.close();
                return;
            }
            CertificateFactory cf = CertificateFactory.getInstance("X.509");
            byte[] certValue = Base64.decodeBase64(certString.getBytes());
            X509Certificate cert = (X509Certificate) cf.generateCertificate(new ByteArrayInputStream(certValue));
            X500Name dn = new X500Name(cert.getSubjectX500Principal().getName());

            String filename = dn.getCommonName().replaceAll(" ", "_");
            response.setContentType("application/x-x509-user-cert");
            response.setHeader("Content-Disposition", "attachment; filename=" + filename + ".crt");

            OutputStream os = response.getOutputStream();
            output = os;
            os.write(cert.getEncoded());
            os.close();
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            response.setContentType("text/html;charset=UTF-8");
            PrintWriter writer = response.getWriter();
            output = writer;
            writer.println("<html>");
            writer.println("<head>");
            writer.println("<title>Download certificate</title>");
            writer.println("</head>");
            writer.println("<body>");
            writer.println("<h1>An error occurred while retrieving the certificate.</h1>");
            writer.println("</body>");
            writer.println("</html>");
            writer.close();
        } finally {
            if (output != null) {
                output.close();
            }
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
