<%--
    Copyright (C) 2012, 2013 Universidad de Murcia

    Authors:
        Ginés Dólera Tormo <ginesdt@um.es>
        Juan M. Marín Pérez <juanmanuel@um.es>
        Jorge Bernal Bernabé <jorgebernal@um.es>
        Gregorio Martínez Pérez <gregorio@um.es>
        Antonio F. Skarmeta Gómez <skarmeta@um.es>

    This file is part of SEMIRAMIS Identity Aggregator (SEMIRAMIS-IA).

    SEMIRAMIS-IA is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    SEMIRAMIS-IA is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with SEMIRAMIS-IA. If not, see <http://www.gnu.org/licenses/>.
--%>

<%@page import="java.util.List"%>
<%@page import="eu.semiramis.identityaggregator.commons.Attribute"%>
<%@page import="eu.semiramis.identityaggregator.RequestorCertInfo"%>

<%@page contentType="text/html" pageEncoding="UTF-8"%>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
        <link rel="stylesheet" type="text/css" href="css/userconsent.css"/>
        <link rel="stylesheet" type="text/css" href="css/basic.css"/>

        <script type="text/javascript" src="js/userconsent.js"></script>
        <script type="text/javascript" src="js/jquery.js"></script>
        <script type="text/javascript" src="js/simplemodal.js"></script>
        <script type="text/javascript" src="js/basic.js"></script>

        <%
            String organization = (String) request.getAttribute("Organization");
            String contactInfo = (String) request.getAttribute("ContactInfo");
            String requestor = (String) request.getAttribute("Requestor");
            RequestorCertInfo requestorCertInfo = (RequestorCertInfo) request.getAttribute("RequestorCertInfo");
            List<Attribute> approvedAttributes = (List<Attribute>) request.getAttribute("ApprovedAttributes");
            List<Attribute> needConsentAttributes = (List<Attribute>) request.getAttribute("NeedConsentAttributes");
        %>
        <title><%= (organization != null) ? organization + " " : ""%>Identity Aggregator</title>
    </head>
    <body>
        <div id="header">
            <img id="semiramis-logo" src="images/logo_semiramis.png" alt="SEMIRAMIS"/>
        </div>
        <div id="consent">
            <div id="intro">
                <p class="welcome-text">Welcome to the Identity Aggregator<% if (organization != null) {%> of the <%= organization%><% }%></p>
                <div id="intro-text">
                    <p>The following attributes have been requested<% if (requestor != null) {%> by <%= requestor%><% }%>.</p>
                    <p>Please, select the attributes you want to release and provide authorization.</p>
                </div>
                <% if (requestorCertInfo != null) {%>
                <a href="#requestorcert" onclick="$('#requestorcert-dialog').modal();">
                    <div id="requestorcert">
                        <table id="requestorcert-table"><tr>
                                <td><img src="images/certificate.png" height="30"/>
                                    <img id="requestorcert-img" src="<% if (requestorCertInfo.isValid()) {%>images/ok.png<% } else {%>images/warning.png<% }%>" height="20"/></td>
                                <td><div id="requestorcert-check">
                                        <% if (requestorCertInfo.isValid()) {%>
                                        <span class="requestorcert-check-ok">Requestor certificate is valid (</span>details<span class="requestorcert-check-ok">)</span>
                                        <% } else {%>
                                        <span class="requestorcert-check-error">Requestor certificate is not valid! (</span>details<span class="requestorcert-check-error">)</span>
                                        <% }%>
                                    </div></td>
                            </tr></table>
                        <div id="requestorcert-dialog" class="basic-dialog">
                            <p class="basic-modal-title">CERTIFICATE PRESENTED<% if (requestor != null) {%> BY <br><strong><%= requestor%></strong><% }%></p>
                            <div id="requestorcert-text" class="basic-modal-text">
                                <p><strong>Version:</strong> <%= requestorCertInfo.getVersion()%></p>
                                <p><strong>Serial Number:</strong> <%= requestorCertInfo.getSerialNumber()%></p>
                                <p><strong>Issuer:</strong> <%= requestorCertInfo.getIssuer()%></p>
                                <p><strong>Subject:</strong> <%= requestorCertInfo.getSubject()%></p>
                                <p><strong>Valid Not Before:</strong> <%= requestorCertInfo.getValidNotBefore()%></p>
                                <p><strong>Valid Not After:</strong> <%= requestorCertInfo.getValidNotAfter()%></p>
                                <p><strong>Certificate Fingerprint</strong></p>
                                <p><%= requestorCertInfo.getFingerprint()%></p>
                                <p><strong>Public Key</strong></p>
                                <p><%= requestorCertInfo.getPublicKeyType()%></p>
                                <p><%= requestorCertInfo.getPublicKey()%></p>
                            </div>
                            <form name="downloadcertform" method="post" action="IAServletDownloadCert">
                                <input name="certificate" type="hidden" value="<%= requestorCertInfo.getEncodedCertificate()%>"/>
                                <div class="basic-modal-footer">
                                    <button name="downloadcert" onclick="downloadcertform.submit()">Download certificate</button>
                                </div>
                            </form>
                        </div>
                    </div>
                </a>
                <% }%>
            </div>
            <div id="attributes-container">
                <form name="approvalForm" method="post" action="IAServletConsent">
                    <input name="approvalDecission" type="hidden" value=""/>
                    <table class="attributes">
                        <tbody>
                            <% for (Attribute attribute : approvedAttributes) {%>
                            <tr>
                                <td class="attr-release-td"><input name="released" value="<%= attribute.getId()%>" type="checkbox" checked<% if (attribute.isRequired()) {%> disabled <% }%>/></td>
                                <td class="attr-name-td"><%= attribute.getId()%> <span class="approved-attr-text">(authorized by policy)</span></td>
                            </tr>
                            <% }%>
                            <% for (Attribute attribute : needConsentAttributes) {%>
                            <tr>
                                <td class="attr-release-td"><input name="released" value="<%= attribute.getId()%>" type="checkbox"<% if (attribute.isRequired()) {%> checked disabled <% }%>/></td>
                                <td class="attr-name-td"><%= attribute.getId()%></td>
                            </tr>
                            <% }%>
                        </tbody>
                    </table>
                </form>
            </div>
            <div id="authorize">
                <p id="privacy-notice-text">By authorizing the attribute release you are accepting the following conditions: <a href="#legalnotice" onclick="$('#legalnotice-dialog').modal();">LEGAL PRIVACY NOTICE</a>.</p>
                <div id="legalnotice-dialog" class="basic-dialog">
                    <p class="basic-modal-title">CONSENT FOR ATTRIBUTE RELEASE<br/><strong>PRIVACY NOTICE</strong></p>
                    <div id="legalnotice-text" class="basic-modal-text">
                        <p>Dear user,</p>
                        <p>To protect your privacy and comply with data protection regulations, the Identity Aggregator<% if (organization != null) {%> at the <%= organization%><% }%> needs your consent to allow the release of your attributes to The Service Provider. Before you make a decision, please be informed that:</p>
                        <p class="basic-modal-text-indent">i) The selected attributes, if they were to be released, would be transferred securely to The Service Provider through the SEMIRAMIS infrastructure, and no SEMIRAMIS component or third party would be able to access them.</p>
                        <p class="basic-modal-text-indent">ii) There exists an agreement between us<% if (organization != null) {%> (<%= organization%>)<% }%> and every Service Provider from which we may receive a request for attributes. Your attributes would only be used for the purpose specified by the Service Provider, and only those attributes which are essential for providing the service have been marked as "mandatory".</p>
                        <p class="basic-modal-text-indent">iii) If you are unsure of which Service Provider requested the attributes, or of which are the conditions that the Service Provider would impose on your data, you are advised NOT to consent to the attribute release and to initiate the process again.</p>
                        <p class="basic-modal-text-indent">iv) If you consent to the attribute release, you are confirming that your information is accurate and up-to-date.</p>
                        <p class="basic-modal-text-indent">v) You may contact us for further information or in the case that you do consent to the attribute release and later want to report a misuse of your data by The Service Provider.<% if (contactInfo != null) {%> You may contact us at <%= contactInfo%>.<% }%></p>
                        <p class="basic-modal-text-indent">vi) You may always contact us to rectify, block, or delete your data at this site. Doing so, however, will have no effect on the attributes released to service providers prior to that action.</p>                        
                        <p>To give your consent, please click on "Authorize" button. If not, click on "Cancel".</p>
                    </div>
                </div>
                <p><input id="accept-conditions" type="checkbox" onClick="toggleDisabled();"/>I accept the conditions in the privacy notice.</p>
                <button id="authorize-button" disabled onclick="submitForm('authorize')">Authorize</button>
                <button id="cancel-button" onclick="submitForm('cancel')">Cancel</button>
            </div>
        </div>
        <div id="footer">
            <div><p class="footer-text"><%= organization%></p></div>                
        </div>
    </body>
</html>
